library IEEE;
	use IEEE.std_logic_1164.all;
	use IEEE.std_logic_unsigned.all;
	use IEEE.std_logic_arith.all;

entity unita_controllo is
	port ( CLOCK : in std_logic;
		   RESET : in std_logic;
		   START : in std_logic;
		   SENSORDATA : in unsigned(11 downto 0);
		   BUSY: out std_logic;
		   PHI1 : out std_logic;
		   SIG1 : out std_logic;
		   SIG2 : out std_logic;
		   PHI2 : out std_logic;
		   AVERAGE : out unsigned(11 downto 0));
end unita_controllo;

architecture A of unita_controllo is

type stato is (idle, s1, s2, s3, s4);
signal current_state, next_state: stato;
signal SELECT_DATA, SELECT_AVERAGE: std_logic;
signal n_cycles, next_cycles: unsigned(3 downto 0);
signal current_sum, sum: unsigned(15 downto 0);

begin
	
	-- Macchina a stati per il controllo del sensore --

	-- Registro di stato --
	process(CLOCK)
	begin
		if CLOCK'event and CLOCK = '1' then
			if RESET = '1' then
				current_state <= idle;
			else
				current_state <= next_state;
			end if;
		end if;
	end process;
	
	--- Processo combinatorio per calcolo uscite e stato futuro ---
	process(current_state, START, SELECT_AVERAGE)
	begin
		case current_state is
	
			when idle => SELECT_DATA <= '0';
						 PHI1 <= '0';
						 SIG1 <= '0';
						 SIG2 <= '0';
						 PHI2 <= '0';
						 BUSY <= '0';
						 if START = '1' then
							next_state <= s1;
						 else
							next_state <= idle;
						 end if;
						 
			when s1 => 	 SELECT_DATA <= '0';
						 PHI1 <= '1';
						 SIG1 <= '1';
						 SIG2 <= '0';
						 PHI2 <= '0';
						 BUSY <= '1';
						 next_state <= s2;
						 
			when s2 =>	 SELECT_DATA <= '0';
						 PHI1 <= '1';
						 SIG1 <= '0';
						 SIG2 <= '1';
						 PHI2 <= '0';
						 BUSY <= '1';
						 next_state <= s3;
			
			when s3 =>	 SELECT_DATA <= '0';
						 PHI1 <= '0';
						 SIG1 <= '0';
						 SIG2 <= '0';
						 PHI2 <= '1';
						 BUSY <= '1';
						 next_state <= s4;
			
			when s4 =>	 SELECT_DATA <= '1';
						 PHI1 <= '0';
						 SIG1 <= '0';
						 SIG2 <= '0';
						 PHI2 <= '0';
						 BUSY <= '1';
						 if SELECT_AVERAGE = '1' then
							next_state <= idle;
						 else
							next_state <= s1;
						 end if;
						 
			when others =>	 SELECT_DATA <= '0';
							 PHI1 <= '0';
							 SIG1 <= '0';
							 SIG2 <= '0';
							 PHI2 <= '0';
							 BUSY <= '0';
							 next_state <= idle;
		end case;
	end process;
	
	-- Registro contatore del numero di volte che vogliamo leggere un valore dal sensore --
	process(CLOCK)
	begin
		if CLOCK'event and CLOCK = '1' then
			if START = '1' or RESET = '1' then
				n_cycles <= conv_unsigned(15, 4);
			elsif SELECT_DATA = '1' then
				n_cycles <= next_cycles;
			end if;
		end if;
	end process;
	
	-- Operazione che mi dice quanti cicli di lettura mi rimangono da compiere --
	next_cycles <= n_cycles - conv_unsigned(1, 4);
	
	-- Ultimo ciclo di lettura --
	SELECT_AVERAGE <= '1' when n_cycles = conv_unsigned(0, 4) else '0';
	
	-- Registro che mi da' la somma dei valori letti dal sensore --
	process(CLOCK)
	begin
		if CLOCK'event and CLOCK = '1' then
			if START = '1' or RESET = '1' then
				current_sum <= conv_unsigned(0, 16);
			elsif SELECT_DATA = '1' then
				current_sum <= sum;
			end if;
		end if;
	end process;
	
	-- Operazione di somma di un nuovo valore letto --
	sum <= current_sum + conv_unsigned(SENSORDATA, 16);
	
	-- Media aritmetica dei valori letti --
	AVERAGE(11 downto 0) <= current_sum(15 downto 4);
	
end A;